# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    HprimDate,
    HprimForeign,
    HprimList,
    HprimMultiple,
    HprimString,
    Segment,
)


class PHDSegmentH(HeaderSegment):
    sender_id = HprimMultiple(
        5, system_name=HprimString(1), instrument_version=HprimString(2)
    )
    processing = HprimString(12, default="P")
    version = HprimString(13, default="LIS-A2")
    date = HprimDate(14)
    patients = HprimForeign("P", "header")
    queries = HprimForeign("Q", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class PHDSegmentP(Segment):
    nom = HprimString((6, 1))
    prenom = HprimString((6, 2))
    orders = HprimForeign("O", "patient")


class PHDSegmentQ(Segment):
    sample_id = HprimString((3, 2))
    test_order = HprimString((5, 4))
    status = HprimDate(13)


class PHDSegmentO(Segment):

    sample_id = HprimString((3, 1))
    worklist_name = HprimString((3, 2))
    position = HprimString((3, 3))
    universal_test_id = HprimList(5, basefield=HprimMultiple(0, test_id=HprimString(4)))
    priority = HprimString(6, default="R")
    date = HprimDate(7)
    action = HprimString(12, default="O")  # Q control
    report = HprimString(26)  # O, Z: no order for patient
    results = HprimForeign("R", "order")


class PHDSegmentR(Segment):
    universal_test_id = HprimMultiple(3, name=HprimString(4), type_resu=HprimString(5))
    value = HprimString(4)
    unit = HprimString(5)
    flag = HprimString((7, 1))
    flag_text = HprimString((7, 2))
    status = HprimString(9)
    date_completed = HprimDate(13)


class PHDSegmentL(Segment):
    code = HprimString(3, default="N")


# N - normal termination.


class Driver(AstmMessageV2):
    SPEC = {"code": "biorad_phd", "label": "Biorad / PHD"}
    SEGMENT_CLASS_MAP = {
        "H": PHDSegmentH,
        "P": PHDSegmentP,
        "O": PHDSegmentO,
        "Q": PHDSegmentQ,
        "R": PHDSegmentR,
        "L": PHDSegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    OPTIONS = [
        AstmMessageV2.Option("phd_qc_prefix", label="Préfixe QC"),
        AstmMessageV2.Option(
            "phd_profil_name",
            label="Code du profil ANA6 pour traitement du résultat particule (champ O.5.4)",
            default="ANA6",
        ),
    ]

    def __init__(self, phd_qc_prefix="", phd_profil_name="ANA6", **kwargs):
        self.phd_qc_prefix = phd_qc_prefix
        self.phd_profil_name = phd_profil_name
        super(Driver, self).__init__(**kwargs)

    def create(self, **kwargs):
        message = PHDSegmentH(type_segment="H", **kwargs)
        message.end = PHDSegmentL(type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        message = self.create(date=context["now"])
        for patient in worklist.get_astm_worklist(
            unique_together=["action"], action={"del": "C", worklist.ELSE: "N"}
        ):
            seg_p = message.patients.add(
                nom=patient["last_name"], prenom=patient["first_name"]
            )
            for sample, action in patient.samples:
                seg_o = seg_p.orders.add(
                    sample_id=sample.sample_id,
                    action=action,
                    date=sample["collection_date"],
                )
                for order in sample.orders:
                    if order["dilution"]:
                        code = "%s,%s" % (order.code, order.dilution)
                    else:
                        code = order.code
                    test = seg_o.universal_test_id.add()
                    test.test_id = code

        if message.patients:
            return message
        elif context["query"]:
            tube = context["query"].queries[0].sample_id
            seg_p = message.patients.add()
            seg_o = seg_p.orders.add(sample_id=tube, action="Y")
            return message

    def decode_message(self, message):
        if message.queries:
            worklist = Worklist(context="query")
            for seg_q in message.queries:
                worklist.get_or_create(
                    type="patient", sample_id=seg_q.sample_id.strip()
                )
            return worklist

        worklist = Worklist(context="result")
        tube_number = ""
        index = 0
        for seg_p in message.patients:
            for seg_o in seg_p.orders:
                qc = seg_o.action == "Q"
                if seg_o.sample_id in ("CAL", "REF"):
                    continue
                if qc or (self.phd_qc_prefix and self.phd_qc_prefix in seg_o.sample_id):
                    sample = worklist.get_or_create(
                        type="control",
                        sample_id=seg_o.sample_id,
                        control_lot=seg_o.sample_id,
                    )
                else:
                    sample = worklist.get_or_create(
                        type="patient",
                        sample_id=seg_o.sample_id,
                        rack="%s-%s" % (seg_o.worklist_name, seg_o.position),
                    )
                # Il s'agit d'un nouveau échantillon
                if (
                    seg_o.sample_id != tube_number
                    or self.phd_profil_name != seg_o.universal_test_id[0].test_id
                ):
                    tube_number = seg_o.sample_id
                    index = 1
                else:
                    # C'est toujours le même échantillon
                    index += 1
                for seg_r in seg_o.results:
                    code = "{}{}{}".format(
                        seg_r.universal_test_id.name,
                        index,
                        seg_r.universal_test_id.type_resu,
                    )
                    r = sample.results.add(
                        code=code,
                        value=seg_r.value,
                        date=seg_r.date_completed or message.date,
                    )
                    if seg_r.flag:
                        r.flags = [seg_r.flag]
                    if seg_r.unit:
                        r.unit = seg_r.unit
        return worklist
