# -*- coding: utf8 -*-
from __future__ import unicode_literals

from collections import OrderedDict

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    Segment,
    HprimString,
    HprimList,
    HprimMultiple,
    HprimDate,
    HprimForeign,
)


class MidisyaAstmSegmentH(HeaderSegment):
    code_emetteur = HprimString(5)
    code_destinataire = HprimString(10)
    date = HprimDate(14)

    patients = HprimForeign("P", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class MidisyaAstmSegmentP(Segment):
    id_patient = HprimString(3)
    nom_patient = HprimString((6, 1))
    prenom_patient = HprimString((6, 2))
    nom_jf = HprimString(7)
    date_naissance = HprimDate(8, fmt_date="%Y%m%d")
    sexe = HprimString(9)
    code_prescripteur = HprimString((14, 1))
    libelle_prescripteur = HprimString((14, 2))
    email_prescripteur = HprimString((14, 3))
    telephone_prescripteur = HprimString((14, 4))
    date_hospitalisation = HprimDate(24)
    code_labo_demandeur = HprimString((26, 1))
    code_correspondant = HprimString((26, 2))
    nom_correspondant = HprimString((26, 3))
    email_correspondant = HprimString((26, 4))
    telephone_correspondant = HprimString((26, 5))
    code_service = HprimString((26, 6))
    nom_service = HprimString((26, 7))
    email_service = HprimString((26, 8))
    telephone_service = HprimString((26, 9))

    orders = HprimForeign("O", "patient")
    orders_request = HprimForeign("S", "patient")


class MidisyaAstmSegmentC(Segment):
    source = HprimString(3)
    commentaire_texte = HprimString(4)
    type_commentaire = HprimString(5)


class MidisyaAstmSegmentO(Segment):
    id_echantillon = HprimString((3, 1))
    rang = HprimString((3, 2))
    isolat = HprimString((3, 3))
    id_dossier = HprimString(4)
    analyses = HprimString((5, 1))
    code_galerie = HprimString((5, 4))
    priorite = HprimString(6)
    date_saisie_dossier = HprimDate((8, 1))
    date_prelevement = HprimDate((8, 2))
    date_reception = HprimDate((8, 3))
    date_ensemencement = HprimDate((8, 4))
    # R : Routine
    # S : Urgent (Stat)
    code_action = HprimString(12)
    # G : Grossesse
    enceinte = HprimString(14)
    nature_echantillon = HprimString(16)
    documents_numerises = HprimList(
        19,
        basefield=HprimMultiple(
            0, code_document=HprimString(1), url_document=HprimString(2)
        ),
    )
    type_resultat = HprimString(25)

    results = HprimForeign("R", "order")


class MidisyaAstmSegmentOrder(MidisyaAstmSegmentO):
    analyses = HprimList(5)

    results = HprimForeign("R", "order")


class MidisyaAstmSegmentR(Segment):
    code_canal = HprimString((3, 1))
    numero_parametre = HprimString((3, 2))
    code_molecule = HprimString((3, 3))
    # Resultats ATB uniquement
    valeur = HprimString((4, 1))
    sensibilite_antibiotique = HprimString((4, 2))
    cmi = HprimString((4, 4))
    code_germe = HprimString((4, 5))
    seuils_cmi = HprimString((4, 6))
    # NM : Numérique
    # HR : Heure (le champ valeur R.4.1 sera alors au format HHMM
    # DT : Date (le champ valeur R.4.1 sera alors au format AAAAMMJJ
    type_resultat = HprimString((4, 9))
    code_utilisateur = HprimString(11)
    date_resultat = HprimDate(13)
    code_automate = HprimString((14, 1))
    libelle_automate = HprimString((14, 2))


class MidisyaAstmSegmentL(Segment):
    code_terminaison = HprimString(3, default="N")


class Driver(AstmMessageV2):
    SPEC = {"code": "midisya", "label": "Midisya via Clarilab"}

    SEGMENT_CLASS_MAP = {
        "H": MidisyaAstmSegmentH,
        "P": MidisyaAstmSegmentP,
        "C": MidisyaAstmSegmentC,
        "O": MidisyaAstmSegmentO,
        "S": MidisyaAstmSegmentOrder,
        "R": MidisyaAstmSegmentR,
        "L": MidisyaAstmSegmentL,
    }

    DEFAULT_SEPARATORS = "|~^&"

    def create(self, **kwargs):
        message = MidisyaAstmSegmentH(type_segment="H", **kwargs)
        message.end = MidisyaAstmSegmentL(type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        message = self.create(date=context["now"])
        for patient in worklist.get_astm_worklist(
            unique_together=["action", "priority"],
            priority={"A": "S", "S": "S", worklist.ELSE: "R"},
            action={"del": "C", "add": "N", "new": "N"},
        ):
            physician = [p for p in patient.physicians if p.type in ("prescriber")]
            seg_p = message.patients.add(
                id_patient=patient.lab_patient_id,
                nom_patient=patient.last_name,
                prenom_patient=patient.first_name,
                nom_jf=patient.maiden_name,
                date_naissance=patient.birthdate,
                sexe=patient.sex,
                date_hospitalisation=patient.admission_date,
                code_prescripteur=physician[0].code if physician else "",
                libelle_prescripteur=physician[0].last_name if physician else "",
                code_labo_demandeur=patient.lab,
                code_service=patient.location,
            )
            if patient.patient_comment1:
                seg_p.commentaires.add(
                    source="H",
                    commentaires=patient.patient_comment1,
                    type_commentaire="G",
                )
            for sample, action, priority in patient.samples:
                seg_o = seg_p.orders_request.add(
                    type_segment="O",
                    id_dossier=sample.request_id,
                    id_echantillon=sample.sample_id,
                    priorite=priority,
                    code_action=action,
                    date_saisie_dossier=sample.request_date,
                    date_prelevement=sample.collection_date,
                    nature_echantillon=sample.specimen,
                )
                if sample.request_comment_patient:
                    seg_p.commentaires.add(
                        source="H",
                        commentaire_texte=sample.request_comment_patient,
                        type_commentaire="G",
                    )
                codes_analyses = []
                for order in sample.orders:
                    codes_analyses.append(order.code)
                    seg_o.analyses = codes_analyses
                if sample.request_comment_obr:
                    seg_o.commentaires.add(
                        source="H",
                        commentaire_texte=sample.request_comment_obr,
                        type_commentaire="G",
                    )
                for result in sample.results:
                    seg_o.results.add(code_canal=result.code, valeur=result.value)
        if message.patients:
            return message

    def decode_message(self, message):
        worklist = Worklist(context="result")
        for seg_p in message.patients:
            comments_atb = []
            for seg_o in seg_p.orders:
                sample = worklist.get_or_create(
                    sample_id=seg_o.id_echantillon, type="patient"
                )
                for seg_c in seg_o.commentaires:
                    if seg_c.type_commentaire == "ATB":
                        comments_atb.append(seg_c.commentaire_texte)
                liste_atb = []
                code_germe = ""
                dict_results = OrderedDict()
                for seg_r in seg_o.results:
                    if seg_r.code_canal == "ID" and seg_o.type_resultat == "ATB":
                        code_germe = seg_r.code_germe
                    if code_germe and seg_r.code_molecule:
                        liste_atb.append(
                            {
                                "antibiotic": "A_{antibiotic}".format(
                                    antibiotic=seg_r.code_molecule.upper()
                                ),
                                "mic": seg_r.cmi,
                                "sensitivity": seg_r.sensibilite_antibiotique,
                                "diametre": seg_r.seuils_cmi,
                            }
                        )
                    # Cytologie results
                    if (
                        not code_germe
                        and seg_r.code_canal
                        and not seg_r.numero_parametre
                        and seg_r.valeur
                    ):
                        sample.results.add(
                            code=seg_r.code_canal,
                            value=seg_r.valeur,
                            date=seg_r.date_resultat,
                            instrument=seg_r.code_automate,
                            operator=seg_r.code_utilisateur,
                        )
                    # Others results
                    if (
                        not code_germe
                        and seg_r.code_canal
                        and seg_r.numero_parametre
                        and seg_r.valeur
                    ):
                        dict_results.setdefault(
                            seg_r.code_canal,
                            {
                                "values": [],
                                "date": seg_r.date_resultat,
                                "instrument": seg_r.code_automate,
                                "operator": seg_r.code_utilisateur,
                            },
                        )
                        dict_results[seg_r.code_canal]["values"].append(seg_r.valeur)
                if dict_results:
                    for code, result in dict_results.items():
                        sample.results.add(
                            code=code,
                            value=";".join(result.get("values")),
                            date=result.get("date"),
                            instrument=result.get("instrument"),
                            operator=result.get("operator"),
                        )
            if liste_atb:
                value = [
                    {
                        "sub_code": seg_o.rang,
                        "antibiogram": liste_atb,
                        "value": "G_{germe}".format(germe=code_germe.upper()),
                    }
                ]
                sample.results.add(
                    code=seg_o.analyses if seg_o.analyses else "RESU_ATB",
                    value=value,
                    date=seg_o.date_saisie_dossier,
                    nom_antibiogramme=seg_o.code_galerie,
                    comments=comments_atb,
                )
        return worklist
