# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    HprimDate,
    HprimForeign,
    HprimList,
    HprimMultiple,
    HprimString,
    Segment,
)


class PantherSegmentH(HeaderSegment):
    sender_id = HprimString(5)
    receiver_id = HprimString(10)
    processing_id = HprimString(12, default="P")
    version = HprimString(13, default="1")
    date = HprimDate(14)

    infos = HprimForeign("M", "infos")
    patients = HprimForeign("P", "patients")
    queries = HprimForeign("Q", "queries")
    end = HprimForeign("L", "header", one2one=True, required=True)


class PantherSegmentQ(Segment):
    specimen_id = HprimString((3, 2))
    universal_test_id = HprimString(5)
    request_information = HprimString(13)


class PantherSegmentP(Segment):
    patient_id = HprimString(3)
    last_name = HprimString((6, 1))
    first_name = HprimString((6, 2))
    birthdate = HprimDate(8, fmt_date="%Y%m%d")
    sex = HprimString(9)
    attending_physician_id = HprimString((14, 1))

    orders = HprimForeign("O", "orders")


class PantherSegmentM(Segment):
    software_version = HprimString(3)
    firmware_version = HprimString(4)
    information = HprimString((5, 1))

    patients = HprimForeign("P", "patients")
    results = HprimForeign("R", "results")


class PantherSegmentO(Segment):
    specimen_id = HprimString(3)
    instrument_id = HprimString((4, 1))
    instrument_specimen = HprimString((4, 2))
    # Un seul demande selon la documentation, il s'agit du profil analytique
    # CT/GC
    universal_test_id = HprimList(
        5,
        basefield=HprimMultiple(
            0,
            code=HprimString(4),
            # GC CT, when empty means all analytes
            analyte_name=HprimString(5),
            # Empty field implies "1" replicate
            number_of_replicate=HprimString(6),
            # Dilution factor = 1
            # Empty field implies factor of "1"
            dilution=HprimString(7),
        ),
    )
    # R: Routine
    # S: Stat
    priority = HprimString(6)
    date_requested = HprimDate(7)
    # Q: Control Sample
    # C: Cancel Request
    # A, N: Panther creates new sample
    action_code = HprimString(12)
    specimen_type = HprimString(16)
    report_type = HprimString(26)

    infos = HprimForeign("M", "infos")
    results = HprimForeign("R", "results")


class PantherSegmentR(Segment):
    assay_short_name = HprimString((3, 4))
    result_aspect = HprimString((3, 5))
    replicate_number = HprimString((3, 6))
    dilution = HprimString((3, 7))
    value = HprimString(4)
    units = HprimString(5)
    abnormal_flag = HprimString(7)
    result_status = HprimList(9)
    date = HprimDate(13)


class PantherSegmentC(Segment):
    # Always I
    comment_source = HprimString(3)
    comment_code = HprimString((4, 1))
    # Always I
    comment_type = HprimString(5)


class PantherSegmentL(Segment):
    # N: Normal
    # I: No information for all queried samples
    terminaison_code = HprimString(3, default="N")


class Driver(AstmMessageV2):

    SPEC = {"code": "panther", "label": "Panther ASTM HOLOGIC"}

    SEGMENT_CLASS_MAP = {
        "H": PantherSegmentH,
        "Q": PantherSegmentQ,
        "P": PantherSegmentP,
        "O": PantherSegmentO,
        "M": PantherSegmentM,
        "R": PantherSegmentR,
        "C": PantherSegmentC,
        "L": PantherSegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    OPTIONS = [
        AstmMessageV2.Option(
            "panther_sender_id",
            label="Id de l'émetteur pour la demande (champ H.5)",
            default="LISHost",
        ),
        AstmMessageV2.Option(
            "panther_receiver_id",
            label="Id du receveur pour la demande (champ H.10)",
            default="Panther",
        ),
    ]

    def __init__(self, **kwargs):
        self.sender_id = kwargs.pop("panther_sender_id", "LISHost")
        self.receiver_id = kwargs.pop("panther_receiver_id", "Panther")
        super(Driver, self).__init__(**kwargs)

    def create(self, **kwargs):
        message = PantherSegmentH(type_segment="H", **kwargs)
        message.end = PantherSegmentL(type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        message = self.create(
            sender_id=self.sender_id, receiver_id=self.receiver_id, date=context["now"]
        )
        for patient in worklist.get_astm_worklist(
            unique_together=["action", "priority"],
            priority={"A": "S", "S": "S", worklist.ELSE: "R"},
            action={"del": "C", "add": "A", "new": "N"},
        ):
            physician = [p for p in patient.physicians if p.type == "attending"]
            seg_p = message.patients.add(
                patient_id=patient.patient_id,
                last_name=patient.last_name,
                first_name=patient.first_name,
                birthdate=patient.birthdate,
                sex=patient.sex,
                attending_physician_id=physician[0].code if physician else "",
            )
            for sample, action, priority in patient.samples:
                seg_o = seg_p.orders.add(
                    specimen_id=sample.sample_id,
                    priority=priority,
                    action_code=action,
                    date_requested=sample.request_date,
                    specimen_type=sample.specimen,
                    report_type="O",
                )
                for order in sample.orders:
                    t = seg_o.universal_test_id.add()
                    t.code = order["code"]
                    t.dilution = order["dilution"]
        if len(message.patients):
            return message
        elif context["query"] and context["query"].queries:
            for seg_q in context["query"].queries:
                seg_p = message.patients.add()
                seg_o = seg_p.orders.add(specimen_id=seg_q.specimen_id, report_type="Y")
                message.end.terminaison_code = "I"
            return message

    def decode_message(self, message):
        worklist = Worklist(context="result")
        if message.queries:
            worklist = Worklist(context="query")
            for seg_q in message.queries:
                worklist.get_or_create(type="patient", sample_id=seg_q.specimen_id)
        else:
            worklist = Worklist(context="result")
            if message.infos:
                patients = message.infos[0].patients
            else:
                patients = message.patients
            for seg_p in patients:
                for seg_o in seg_p.orders:
                    if seg_o.action_code == "Q":
                        sample = worklist.get_or_create(
                            sample_id=seg_o.specimen_id,
                            control_lot=seg_o.specimen_id,
                            specimen=seg_o.specimen_type,
                            type="control",
                        )
                    else:
                        sample = worklist.get_or_create(
                            sample_id=seg_o.specimen_id,
                            specimen=seg_o.specimen_type,
                            type="patient",
                        )
                    if seg_o.infos:
                        results = seg_o.infos[0].results
                    else:
                        results = seg_o.results
                    for seg_r in results:
                        flags = []
                        for seg_c in seg_r.commentaires:
                            flags.append(seg_c.comment_code)
                        if seg_r.abnormal_flag:
                            flags.append(seg_r.abnormal_flag)
                        sample.results.add(
                            code="{}{}".format(
                                seg_r.assay_short_name, seg_r.result_aspect
                            ),
                            value=seg_r.value,
                            flags=flags,
                            dilution=seg_r.dilution,
                            unit=seg_r.units,
                            date=seg_r.date,
                        )
        return worklist
